package org.eclipse.jdt.internal.compiler.util;

package .tmp;

import org.eclipse.jdt.core.compiler.CharOperation;

public final class SimpleSetOfCharArray implements Cloneable {
  public char[][] values;
  
  public int elementSize;
  
  public int threshold;
  
  public SimpleSetOfCharArray() {
    this(13);
  }
  
  public SimpleSetOfCharArray(int size) {
    if (size < 3)
      size = 3; 
    this.elementSize = 0;
    this.threshold = size + 1;
    this.values = new char[2 * size + 1][];
  }
  
  public Object add(char[] object) {
    int length = this.values.length;
    int index = (CharOperation.hashCode(object) & Integer.MAX_VALUE) % length;
    char[] current;
    while ((current = this.values[index]) != null) {
      if (CharOperation.equals(current, object)) {
        this.values[index] = object;
        return object;
      } 
      if (++index == length)
        index = 0; 
    } 
    this.values[index] = object;
    if (++this.elementSize > this.threshold)
      rehash(); 
    return object;
  }
  
  public void asArray(Object[] copy) {
    if (this.elementSize != copy.length)
      throw new IllegalArgumentException(); 
    int index = this.elementSize;
    for (int i = 0, l = this.values.length; i < l && index > 0; i++) {
      if (this.values[i] != null)
        copy[--index] = this.values[i]; 
    } 
  }
  
  public void clear() {
    for (int i = this.values.length; --i >= 0;)
      this.values[i] = null; 
    this.elementSize = 0;
  }
  
  public Object clone() throws CloneNotSupportedException {
    org.eclipse.jdt.internal.compiler.util.SimpleSetOfCharArray result = (org.eclipse.jdt.internal.compiler.util.SimpleSetOfCharArray)super.clone();
    result.elementSize = this.elementSize;
    result.threshold = this.threshold;
    int length = this.values.length;
    result.values = new char[length][];
    System.arraycopy(this.values, 0, result.values, 0, length);
    return result;
  }
  
  public char[] get(char[] object) {
    int length = this.values.length;
    int index = (CharOperation.hashCode(object) & Integer.MAX_VALUE) % length;
    char[] current;
    while ((current = this.values[index]) != null) {
      if (CharOperation.equals(current, object))
        return current; 
      if (++index == length)
        index = 0; 
    } 
    this.values[index] = object;
    if (++this.elementSize > this.threshold)
      rehash(); 
    return object;
  }
  
  public boolean includes(char[] object) {
    int length = this.values.length;
    int index = (CharOperation.hashCode(object) & Integer.MAX_VALUE) % length;
    char[] current;
    while ((current = this.values[index]) != null) {
      if (CharOperation.equals(current, object))
        return true; 
      if (++index == length)
        index = 0; 
    } 
    return false;
  }
  
  public char[] remove(char[] object) {
    int length = this.values.length;
    int index = (CharOperation.hashCode(object) & Integer.MAX_VALUE) % length;
    char[] current;
    while ((current = this.values[index]) != null) {
      if (CharOperation.equals(current, object)) {
        this.elementSize--;
        char[] oldValue = this.values[index];
        this.values[index] = null;
        if (this.values[(index + 1 == length) ? 0 : (index + 1)] != null)
          rehash(); 
        return oldValue;
      } 
      if (++index == length)
        index = 0; 
    } 
    return null;
  }
  
  private void rehash() {
    org.eclipse.jdt.internal.compiler.util.SimpleSetOfCharArray newSet = new org.eclipse.jdt.internal.compiler.util.SimpleSetOfCharArray(this.elementSize * 2);
    for (int i = this.values.length; --i >= 0;) {
      if ((current = this.values[i]) != null)
        newSet.add(current); 
    } 
    this.values = newSet.values;
    this.elementSize = newSet.elementSize;
    this.threshold = newSet.threshold;
  }
  
  public String toString() {
    String s = "";
    for (int i = 0, l = this.values.length; i < l; i++) {
      char[] object;
      if ((object = this.values[i]) != null)
        s = String.valueOf(s) + new String(object) + "\n"; 
    } 
    return s;
  }
}
